//
//  FileManager.m
//  AsReaderScan
//
//  Created by Yasutaka Oshiro on 2021/08/31.
//

#import "FileManager.h"
#import "GlobalDefine.h"
#import <xlsxwriter/xlsxwriter.h>

@implementation FileManager

typedef NS_ENUM(NSInteger, ExportType){
    ExportTypeBarcode,
    ExportTypeRFID,
    ExportTypeNFC
};

static FileManager *instance = nil;

+ (instancetype)sharedInstance
{
   @synchronized(self) {
       if (!instance) {
           instance = [FileManager new];
       }
   }
   return instance;
}

#pragma mark - Private Method
//----------------------------------------------------------------------------------------
- (NSString *)filePath:(NSString *)fileName
{
    NSArray *paths = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES);
    NSString *path = [paths objectAtIndex:0];
    NSString *filePath = [path stringByAppendingPathComponent:fileName];
    return filePath;
}

- (BOOL)isFileExist:(NSString *)fileName
{
    NSFileManager *fileManager = [NSFileManager defaultManager];
    BOOL result = [fileManager fileExistsAtPath:[self filePath:fileName]];
    return result;
}

- (void)createCSVWithFileName:(NSString *)fileName
{
    NSFileManager *fileManager = [NSFileManager defaultManager];
    if (![fileManager createFileAtPath:[self filePath:fileName]  contents:nil attributes:nil]) {
    }
}

#pragma mark - Public Method
//----------------------------------------------------------------------------------------
- (BOOL)exportExcel:(NSString *)fileName dataArray:(NSMutableArray *)dataArray
{
    NSFileManager *fileManager = [NSFileManager defaultManager];
    NSString *filePath = [self filePath:fileName];
    BOOL result = [fileManager fileExistsAtPath:filePath];
    
    if ( result==YES ) {
        [fileManager removeItemAtPath:filePath error:nil];
    }
    
    if ( [fileManager createFileAtPath:filePath contents:nil attributes:nil]==NO ) {
        return  NO;
    }

    // 创建新xlsx文件，路径需要转成c字符串
    lxw_workbook *workbook = workbook_new([filePath UTF8String]);
    // 创建sheet
    lxw_worksheet *worksheet = workbook_add_worksheet(workbook, NULL);
    //设置格式
    lxw_format *liftformat = workbook_add_format(workbook);
    format_set_font_color(liftformat, LXW_COLOR_BLACK);
    format_set_align(liftformat, LXW_ALIGN_LEFT);

    int i = 1;
    for (NSDictionary *inserDic in dataArray ) {
        int columnValue1 = 0;
        int columnValue2 = 0;
        int columnValue3 = 0;
        BOOL writeFlag = NO;
        char *name;
        
        if ( [fileName hasPrefix:@"barcode"]==YES ) {
            columnValue1 = 20;
            columnValue2 = 25;
            columnValue3 = 10;
            name = "barcode";
            writeFlag = YES;
            
        } else if( [fileName hasPrefix:@"rfid"]==YES ) {
            columnValue1 = 35;
            columnValue2 = 25;
            columnValue3 = 10;
            name = "PCEPC Hex";
            writeFlag = YES;
            
        } else if( [fileName hasPrefix:@"nfc"]==YES ) {
            columnValue1 = 20;
            columnValue2 = 25;
            columnValue3 = 10;
            name = "NFC";
            writeFlag = YES;
        }
        
        if( writeFlag==YES ){
            // 设置列宽
            worksheet_set_column(worksheet, COLS("A:A"), columnValue1, NULL);
            worksheet_set_column(worksheet, COLS("B:B"), columnValue2, NULL);
            worksheet_set_column(worksheet, COLS("C:C"), columnValue3, NULL);
            //第一行
            worksheet_write_string(worksheet, 0, 0, (const char *)name, liftformat);
            worksheet_write_string(worksheet, 0, 1, "time", liftformat);
            worksheet_write_string(worksheet, 0, 2, "count", liftformat);

            char tagChar[65535];
            char dateTimeChar[1000];
            char countChar[65535];
            strcpy(tagChar,(char *)[[inserDic objectForKey:df_CELL_TAG_INFO] UTF8String]);
            strcpy(dateTimeChar,(char *)[[inserDic objectForKey:df_CELL_TAG_DATETIME] UTF8String]);
            strcpy(countChar,(char *)[[inserDic objectForKey:df_CELL_TAG_COUNT] UTF8String]);
            worksheet_write_string(worksheet, i, 0, tagChar, liftformat);
            worksheet_write_string(worksheet, i, 1, dateTimeChar, liftformat);
            worksheet_write_string(worksheet, i, 2, countChar, liftformat);
            i ++;
        }
    }
    workbook_close(workbook);
    return  YES;
}

- (BOOL)exportCSV:(NSString *)fileName dataArray:(NSArray *)dataArray
{
    NSFileManager *fileManager = [NSFileManager defaultManager];
    NSString *filePath = [self filePath:fileName];
    
    if( [fileManager fileExistsAtPath:filePath]==YES ) {
        [fileManager removeItemAtPath:filePath error:nil];
    }
    if ([fileManager createFileAtPath:filePath contents:nil attributes:nil]==NO) {
        return NO;
    }
    
    NSOutputStream *output = [[NSOutputStream alloc] initToFileAtPath:[self filePath:fileName] append:YES];
    [output open];
    
    if ( [output hasSpaceAvailable]==NO ) {
        return NO;
    } else {
        NSString *header;
        if ([fileName hasPrefix:@"barcode"]) {
            header = @"\"barcode\",\"time\",\"count\"\n";
        } else if ([fileName hasPrefix:@"rfid"]) {
            header = @"\"pcepc hex\",\"time\",\"count\"\n";
        } else if ([fileName hasPrefix:@"nfc"]) {
            header = @"\"nfc\",\"time\",\"count\"\n";
        }
        const uint8_t *headerString = (const uint8_t *)[header cStringUsingEncoding:NSUTF8StringEncoding];
        NSInteger headerLength = [header lengthOfBytesUsingEncoding:NSUTF8StringEncoding];
        NSInteger result = [output write:headerString maxLength:headerLength];
        if (result <= 0) {
            return NO;
        }
        
        for( NSDictionary *inserDic in dataArray ) {
            
            if( [fileName hasPrefix:@"barcode"]==YES ||
                [fileName hasPrefix:@"rfid"]==YES ||
                [fileName hasPrefix:@"nfc"]==YES ) {
                
                NSString *row = [NSString stringWithFormat:@"\"%@\",\"%@\",\"%@\"\n", [inserDic objectForKey:df_CELL_TAG_INFO], [inserDic objectForKey:df_CELL_TAG_DATETIME], [inserDic objectForKey:df_CELL_TAG_COUNT]];
                const uint8_t *rowString = (const uint8_t *)[row cStringUsingEncoding:NSUTF8StringEncoding];
                NSInteger rowLength = [row lengthOfBytesUsingEncoding:NSUTF8StringEncoding];
                result = [output write:rowString maxLength:rowLength];
                
                if (result <= 0) {
                    [output close];
                    return NO;
                }
            }
        }
        [output close];
    }
    return YES;
}

- (void)deleteFile:(NSString *)fileName
{
    NSFileManager *fileManager = [NSFileManager defaultManager];
    [fileManager removeItemAtPath:[self filePath:fileName] error:nil];
}

@end
